package fi.fa.opplugin.utils;

import com.alibaba.druid.util.StringUtils;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import org.apache.commons.lang3.ObjectUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class ChangeDataUtils {
    /**
     * @description 查询财务卡片
     * @param realCardId
     */
    public static DynamicObject queryFinCard(Long realCardId) {
        QFilter cardFilter = new QFilter("realcardmasterid", QCP.equals,realCardId);
        DynamicObject[] card = BusinessDataServiceHelper.load("fa_card_fin","id",new QFilter[] {cardFilter}, "endperiod.id DESC");
        for (int c = 0; c < card.length; c++) {
            DynamicObject cardInfo = BusinessDataServiceHelper.loadSingle(card[0].getPkValue(), card[0].getDynamicObjectType().getName());
            return cardInfo;
        }
        return null;
    }

    /**
     * @description 查询折旧用途
     * @param id
     */
    public static DynamicObject queryDepreuse(Long id) {
        QFilter orgQFilter= new QFilter("id", QCP.equals,id);
        DynamicObject[] depreuse = BusinessDataServiceHelper.load("fa_depreuse","id",new QFilter[] {orgQFilter});
        for (int c = 0; c < depreuse.length; c++) {
            DynamicObject depreuseInfo = BusinessDataServiceHelper.loadSingle(depreuse[c].getPkValue(), depreuse[c].getDynamicObjectType().getName());
            return depreuseInfo;
        }
        return null;
    }

    /**
     * @description 查询变更类型
     * @param id
     */
    public static DynamicObject queryChangeType(Long id) {
        QFilter orgQFilter= new QFilter("id", QCP.equals,id);
        DynamicObject[] changeType = BusinessDataServiceHelper.load("fa_change_type","id",new QFilter[] {orgQFilter});
        for (int c = 0; c < changeType.length; c++) {
            DynamicObject changeInfo = BusinessDataServiceHelper.loadSingle(changeType[c].getPkValue(), changeType[c].getDynamicObjectType().getName());
            return changeInfo;
        }
        return null;
    }

    /**
     * @description 查询组织
     * @param orgId
     */
    public static DynamicObject queryOrg(Long orgId) {
        QFilter orgQFilter= new QFilter("id", QCP.equals,orgId);
        DynamicObject[] org = BusinessDataServiceHelper.load("bos_org","id",new QFilter[] {orgQFilter});
        for (int c = 0; c < org.length; c++) {
            DynamicObject orgInfo = BusinessDataServiceHelper.loadSingle(org[c].getPkValue(), org[c].getDynamicObjectType().getName());
            return orgInfo;
        }
        return null;
    }

    /**
     * @description 查询组织对照表获取组织id
     * @param eOrgId
     * @return Long
     */
    public static Long getOrg(String eOrgId) {
        QFilter orgFilter = new QFilter("nckd_e_org", QCP.equals,eOrgId);
        DynamicObject[] orgControl = BusinessDataServiceHelper.load("nckd_base_org_ext","id",new QFilter[] {orgFilter});
        if (orgControl.length > 0) {
            DynamicObject orgControlInfo = BusinessDataServiceHelper.loadSingle(orgControl[0].getPkValue(), orgControl[0].getDynamicObjectType().getName());
            String number = orgControlInfo.getString("nckd_org_number");
            QFilter filter = new QFilter("number", QCP.equals,number);
            DynamicObject[] org = BusinessDataServiceHelper.load("bos_org","id",new QFilter[] {filter});
            if (org.length > 0) {
                DynamicObject orgInfo = BusinessDataServiceHelper.loadSingle(org[0].getPkValue(), org[0].getDynamicObjectType().getName());
                return orgInfo.getLong("id");
            }
        }
        return null;
    }

    /**
     * @description 查询业务单元id
     * @param id
     * @return Long
     */
    public static Long getRelation(Long id) {

        QFilter filter = new QFilter("toorg", QCP.equals,id);
        DynamicObject[] org = BusinessDataServiceHelper.load("bos_org_orgrelation_dept","id",new QFilter[] {filter});
        if (org.length > 0) {
            DynamicObject orgInfo = BusinessDataServiceHelper.loadSingle(org[0].getPkValue(), org[0].getDynamicObjectType().getName());
            DynamicObject ywdyOrg = orgInfo.getDynamicObject("fromorg");
            if (ywdyOrg != null) {
                return ywdyOrg.getLong("id");
            }
        }
        return null;
    }

    /**
     * @description 查询用户表获取用户
     * @param eUser
     * @return Long
     */
    public static DynamicObject getUser(String eUser) {
        QFilter filter = new QFilter("number", QCP.in, eUser);
        DynamicObject[] user = BusinessDataServiceHelper.load("bos_user","id",new QFilter[] {filter});
        if (user.length > 0) {
            DynamicObject userInfo = BusinessDataServiceHelper.loadSingle(user[0].getPkValue(), user[0].getDynamicObjectType().getName());
            return userInfo;
        }
        return null;
    }

    /**
     * @description e管家推送数据校验
     * @param info
     * @return String
     */
    public static String check(DynamicObject info) {
        String billno = info.getString("number");//源单编号
        String dispatchType = info.getString("nckd_dispatch_type");//调拨类型
        String nckd_card_real_code = info.getString("nckd_card_number");
        String[] asset = nckd_card_real_code.split(",");
        if (nckd_card_real_code.isEmpty()) {
            return "单据" + billno + "未绑定资产信息,生成调拨单失败";
        }
        // 使用 Stream 判断 : 后的数据是否相同
        boolean isSame = Arrays.stream(asset)
                .map(item -> item.split(":")[1]) // 提取组织部分
                .distinct()                       // 去重
                .count() <= 1;                   // 如果去重后的数量小于等于 1，则表示相同
        // 使用 Stream 获取所有的 ID（前面的数据）
        List<String> numberList = Arrays.stream(asset)
                .map(item -> item.split(":")[0])  // 分割并提取 ID
                .collect(Collectors.toList());      // 收集到列表中

        //组织相同则处理
        if (isSame) {
            //经费系统实物卡片编码
            List<String> cardNumberList = new ArrayList<>();
            //经费系统实物卡片关联的e管家实物卡片编码
            List<String> eCardNumberList = new ArrayList<>();
            //numberList 中有而 eCardNumberList 中没有的元素
            List<String> missingInECard = new ArrayList<>();
            String e_orgId = Arrays.stream(asset).map(item -> item.split(":")[1]).collect(Collectors.toList()).get(0);
            //查询e管家实物卡片清单
            QFilter filter = new QFilter("number", QCP.in,numberList);
            DynamicObject[] dynamicObjects = BusinessDataServiceHelper.load("nckd_fa_card_real_list","id,nckd_card_real_code",new QFilter[] {filter});
            //查询e管家实物卡片关联的经费系统实物卡片
            for (DynamicObject eCard : dynamicObjects) {
                String cardNumber = eCard.getString("nckd_card_real_code");
                cardNumberList.add(cardNumber);
            }
            //再查经费系统实物卡片关联的e管家实物卡片
            QFilter filter1 = new QFilter("nckd_card_real_code", QCP.in,cardNumberList);
            DynamicObject[] edynamicObjects = BusinessDataServiceHelper.load("nckd_fa_card_real_list","id",new QFilter[] {filter1});
            //获取e管家实物卡片编码
            for (DynamicObject eCard : edynamicObjects) {
                String number = eCard.getString("number");
                eCardNumberList.add(number);
            }
            //对比中间表关联的e管家实物卡片和查询到的e管家实物卡片
            if (eCardNumberList.size() > numberList.size()) {
                return "单据"+billno+"绑定的资产有未同步至星瀚的数据,生成调拨单失败";
            } else if (eCardNumberList.size() == numberList.size()) {
                boolean areEqual = areListsEqual(numberList, eCardNumberList);
                if (!areEqual) {
                    return "单据"+billno+"绑定的资产与经费系统实物卡片绑定的资产不一致,生成调拨单失败";
                }
            }else {
                // 检查 eCardNumberList 是否有 numberList 没有的元素
                for (String eCard : eCardNumberList) {
                    if (!numberList.contains(eCard)) {
                        return "单据"+billno+"绑定的资产与经费系统实物卡片绑定的资产不一致,生成调拨单失败";
                    }
                }

                for (String number : numberList) {
                    if (!eCardNumberList.contains(number)) {
                        missingInECard.add(number);
                    }
                }
            }
            //查询未匹配到的e管家实物卡片做更新
            QFilter filter2 = new QFilter("number", QCP.in,missingInECard);
            DynamicObject[] missingObject = BusinessDataServiceHelper.load("nckd_fa_card_real_list","id",new QFilter[] {filter2});
            Long deptId = getOrg(info.getString("nckd_dept_in"));
            Long orgId = getRelation(deptId);
            for (DynamicObject object : missingObject) {
                DynamicObject eCardInfo = BusinessDataServiceHelper.loadSingle(object.getPkValue(), object.getDynamicObjectType().getName());
                if ("0".equals(dispatchType)) {
                    String nckd_storeplace = info.getString("nckd_storeplace");
                    String user = info.getString("nckd_used_user");
                    //调出单更新组织部门人员存放地点
                    eCardInfo.set("nckd_orgfield", orgId);
                    eCardInfo.set("nckd_dept", deptId);
                    eCardInfo.set("nckd_local", nckd_storeplace);
                    eCardInfo.set("nckd_use_name", user);

                } else if ("1".equals(dispatchType)) {
                    String nckd_storeplace = info.getString("nckd_storeplace");
                    String user = info.getString("nckd_used_user");
                    //变更单更新部门人员存放地点
                    eCardInfo.set("nckd_dept", deptId);
                    eCardInfo.set("nckd_local", nckd_storeplace);
                    eCardInfo.set("nckd_use_name", user);
                }else {
                    //更新是否报废为是
                    eCardInfo.set("nckd_discard", 1);
                }
                SaveServiceHelper.save(new DynamicObject[]{eCardInfo});
            }
        }else {
            //资产信息组织不同
            return "单据"+billno+"绑定的资产信息组织不相同,生成调拨单失败";
        }
        return null;
    }

    public static List<String> getCardNumber(DynamicObject info) {
        String nckd_card_real_code = info.getString("nckd_card_number");
        String[] asset = nckd_card_real_code.split(",");
        List<String> codeList = new ArrayList<>();
        List<String> numberList = Arrays.stream(asset).map(item -> item.split(":")[0]).collect(Collectors.toList());
        QFilter listFilter = new QFilter("number", QCP.in,numberList);
        DynamicObject[] cardList = BusinessDataServiceHelper.load("nckd_fa_card_real_list","nckd_card_real_code",new QFilter[] {listFilter});
        for (int i = 0; i < cardList.length; i++) {
            DynamicObject cardInfo = BusinessDataServiceHelper.loadSingle(cardList[i].getPkValue(), cardList[i].getDynamicObjectType().getName());
            String code = cardInfo.getString("nckd_card_real_code");
            codeList.add(code);
        }
        return codeList;
    }

    public static boolean areListsEqual(List<String> list1, List<String> list2) {
        for (int i = 0; i < list1.size(); i++) {
            if (!list1.get(i).equals(list2.get(i))) {
                return false; // 发现元素不相同
            }
        }
        return true; // 所有元素相同
    }

}
